﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.Linq;
using Kreta.BusinessLogic.Helpers;
using Kreta.Client.Tavollet;
using Kreta.Core.Configuratiaton;
using Kreta.Core.SAP.CommunicationModels.SzamfejtettTavolletek;
using Kreta.Enums;
using Kreta.Web.Areas.HRModul.Models;
using Kreta.Web.Helpers;
using Kreta.Web.Security;

namespace Kreta.Web.Areas.HRModul.Logic
{
    public class TavolletListaLogic
    {
        public TavolletListaSearchModel GetTavolletListaSearchModel()
        {
            var helper = new HRModulHelper(ConnectionTypeExtensions.GetActiveSessionConnectionType());
            TavolletListaSearchModel model = new TavolletListaSearchModel()
            {
                SrcIsLeaderCall = helper.IsLeader(),
            };

            var tanev = new TanevHelper(ConnectionTypeExtensions.GetSessionConnectionType());
            var tanevCo = tanev.GetTanevInfo();

            var kozpontiKreta = (KozpontiKretaConfiguration)ConfigurationManager.GetSection("KozpontiKretaConfig");
            var client = new TavolletClient(kozpontiKreta.KgrUrl, kozpontiKreta.ApiKey);

            if (!ClaimData.IsDefaultAdminUser)
            {
                var kozpontiResult = client.GetTavolletAlkalmazottakAdatai(model.ConvertToTavolletAdatokRequestModel(tanevCo.Sorszam, ClaimData.FelhasznaloId));

                if (kozpontiResult.IsSuccess && kozpontiResult.TavolletAdatokResultList != null && kozpontiResult.TavolletAdatokResultList.Count > 0)
                {
                    model.SzTSzAzonosito = kozpontiResult.TavolletAdatokResultList[0].SzTSzAzonosito;
                    model.SzabadsagKeret = kozpontiResult.TavolletAdatokResultList[0].EvesKeret;
                    model.IgenybeVehetoTavolletek = kozpontiResult.TavolletAdatokResultList[0].FelhasznalhatoKeret;
                    model.IgenybeVettTavolletek = kozpontiResult.TavolletAdatokResultList[0].FelhasznaltKeret;
                    model.FuggobenLevoTavolletek = kozpontiResult.TavolletAdatokResultList[0].FuggoTavolletIgenyek;
                    model.UtolsoSzinkron = kozpontiResult.TavolletAdatokResultList[0].UtolsoSzinkron;
                    model.IsTavolletFoIntezmenye = kozpontiResult.TavolletAdatokResultList[0].IsTavolletFoIntezmenye;
                }
            }
            else
            {
                model.IsTavolletFoIntezmenye = true;
            }

            if ((ClaimData.FelhasznaloSzerepkor == Enums.SzerepkorTipusEnum.Adminisztrator) || helper.IsHrKezelo())
                model.SrcIsAdminCall = true;

            if (!model.SrcIsLeaderCall && !model.SrcIsAdminCall)
            {
                model.SrcAlkalmazott = new int[] { ClaimData.FelhasznaloId };
                model.AlkalmazottFilterId = ClaimData.FelhasznaloId;
            }

            model.MinimumSrcIdoszakKezdet = model.MinimumSrcIdoszakVeg = helper.GetMinimumNaptariEv(ClaimData.IsAdministrator);

            model.SrcNaptariEv = new int[] { DateTime.Now.Year };

            return model;
        }

        public (IntezmenyiTavolletBefogadottResponseModel, List<string>) GetTavolletBefogadottLista(TavolletListaSearchModel model, HRModulHelper helper, AlkalmazottHelper alkalmazottHelper)
        {
            IntezmenyiTavolletBefogadottResponseModel kozpontiResult = null;
            List<string> sztszAzonositok = null;

            var isLeader = helper.IsLeader();

            if (!isLeader && ClaimData.FelhasznaloSzerepkor != SzerepkorTipusEnum.Adminisztrator && !helper.IsHrKezelo())
            {
                model.SrcAlkalmazott = new int[] { ClaimData.FelhasznaloId };
            }
            else
            {
                if (model.SrcAlkalmazott == null || model.SrcAlkalmazott.Length < 1)
                {
                    model.SrcAlkalmazott = alkalmazottHelper.GetAlkalmazottIds(null).ToArray();
                }
                else
                {
                    model.SrcAlkalmazott = model.SrcAlkalmazott.Intersect(alkalmazottHelper.GetAlkalmazottIds(null)).ToArray();
                }
            }

            if (model.SrcAlkalmazott.Length > 0)
            {
                var alkalmazottByAlkalmazottId = alkalmazottHelper.GetAlkalmazottIDTavolletAlkalmazott(model.SrcAlkalmazott.ToList());

                sztszAzonositok = alkalmazottByAlkalmazottId.Values.Where(x => !string.IsNullOrWhiteSpace(x.AlkalmazottSzTSz)).Select(x => x.AlkalmazottSzTSz).Distinct().ToList();

                var kozpontiKreta = (KozpontiKretaConfiguration)ConfigurationManager.GetSection("KozpontiKretaConfig");
                var client = new TavolletClient(kozpontiKreta.KgrUrl, kozpontiKreta.ApiKey);
                var requestModel = new IntezmenyiTavolletBefogadottRequestModel
                {
                    SzTSzAzonositok = sztszAzonositok,
                    TavolletRogzitesEvek = (model.SrcNaptariEv != null && model.SrcNaptariEv.Length > 0) ? model.SrcNaptariEv.ToList() : new List<int> { DateTime.Now.Year },
                    TavolletKezdete = model.SrcIdoszakKezdet,
                    TavolletVege = model.SrcIdoszakVeg,
                    TavolletTipusId = model.SrcTipus
                };

                if (!isLeader && ClaimData.FelhasznaloSzerepkor != SzerepkorTipusEnum.Adminisztrator && !helper.IsHrKezelo())
                {
                    kozpontiResult = client.GetAlkalmazottakTavolletAdatok(requestModel);
                }
                else
                {
                    kozpontiResult = client.GetVezetokTavolletAdatok(requestModel);
                }
            }

            return (kozpontiResult, sztszAzonositok);
        }
    }
}
